package main

import (
	"context"
	"database/sql"
	"log"
	"os"
	"time"

	"git.netflux.io/rob/clipper/generated/store"
	"git.netflux.io/rob/clipper/server"
	"github.com/aws/aws-sdk-go-v2/config"
	"github.com/aws/aws-sdk-go-v2/service/s3"
	"github.com/kkdai/youtube/v2"
	_ "github.com/lib/pq"
)

const (
	DefaultHTTPBindAddr = "0.0.0.0:8888"
	DefaultTimeout      = 600 * time.Second
)

func main() {
	ctx := context.Background()

	// Create a store
	databaseURL := os.Getenv("DATABASE_URL")
	if databaseURL == "" {
		log.Fatal("DATABASE_URL not set")
	}

	db, err := sql.Open("postgres", databaseURL)
	if err != nil {
		log.Fatal(err)
	}
	store := store.New(db)

	// Create an Amazon S3 service s3Client
	cfg, err := config.LoadDefaultConfig(ctx)
	if err != nil {
		log.Fatal(err)
	}
	s3Client := s3.NewFromConfig(cfg)

	// Create a Youtube client
	var youtubeClient youtube.Client

	serverOptions := server.Options{
		BindAddr:      DefaultHTTPBindAddr,
		Timeout:       DefaultTimeout,
		Store:         store,
		YoutubeClient: &youtubeClient,
		S3Client:      s3Client,
	}

	log.Fatal(server.Start(serverOptions))
}
